/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programs/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skCentralWidget.cpp 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

/*------------------------------------------------------------- Includes */

#include <qapplication.h>
#include <qwidget.h>
#include <qhbox.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qsplitter.h>
#include <qlistview.h>
#include <qlineedit.h>
#include <qpopupmenu.h>
#include <qcheckbox.h>
#include <qprogressdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qpaintdevicemetrics.h>
#include <qsimplerichtext.h>


#include "skCentralWidget.h"
#include "skUncompress.h"
#include "skGlobal.h"
#include "skSegorSubCategory.h"
#include "skSegorSubSubCategory.h"
#include "skSegorArticle.h"
#include "skTextCell.h"

#include <iostream>
#include <fstream>

using namespace std;

/*-------------------------------------------------------------- Const values */

/** Number of catalog files to read */
const int CATALOGFILES = 4;

/*------------------------------------------------------ Class functions */

/** Constructor for the \a skCentralWidget class.
* @param parent Pointer to the parent widget
* @param name Internal name for the new \a skCentralWidget widget
*/
skCentralWidget::skCentralWidget( QWidget *parent, const char *name )
    : QVBox( parent, name)
{
  setupCentralWidget();
  setupConnects();
}


/** Sets up the widgets for the main window area. */
void skCentralWidget::setupCentralWidget()
{
  QSplitter *split = new QSplitter(this, "split");

  QVBox *vbLeft = new QVBox(split, "vbLeft");
  lvParts = new QListView(split, "lvParts");
  
  // Set up left half of the screen
  QVBox *vbLUpper = new QVBox(vbLeft, "vbLUpper");
  QVBox *vbLLower = new QVBox(vbLeft, "vbLLower");
  QVBox *vbLSearch = new QVBox(vbLeft, "vbLSearch");

  QGroupBox *gbCategory = new QGroupBox(1, QGroupBox::Horizontal, tr("Warenhauptgruppe"), vbLUpper, "gbCategory");
  cbCategory = new QComboBox(gbCategory, "cbCategory");
  QGroupBox *gbSubCategory = new QGroupBox(1, QGroupBox::Horizontal, tr("Warenobergruppe"), vbLUpper, "gbSubCategory");
  cbSubCategory = new QComboBox(gbSubCategory, "cbSubCategory");
  QGroupBox *gbSubSubCategory = new QGroupBox(1, QGroupBox::Horizontal, tr("Warengruppe"), vbLUpper, "gbSubSubCategory");
  cbSubSubCategory = new QComboBox(gbSubSubCategory, "cbSubSubCategory");

  QGroupBox *gbSearch = new QGroupBox(4, QGroupBox::Vertical, tr("Suchen"), vbLLower, "gbSearch");
  QGroupBox *gbSearchString = new QGroupBox(1, QGroupBox::Horizontal, tr("Nach"), gbSearch, "gbSearchString");
  leSearch = new QLineEdit(gbSearchString, "leSearch");
  QGroupBox *gbSearchIn = new QGroupBox(3, QGroupBox::Vertical, tr("In"), gbSearch, "gbSearchIn");
  cbName = new QCheckBox(tr("Artikelname"), gbSearchIn, "cbName");
  cbName->setChecked(true);
  cbDescription = new QCheckBox(tr("Beschreibung"), gbSearchIn, "cbDescription");
  cbDescription->setChecked(true);
  cbOtherNames = new QCheckBox(tr("Weitere Namen"), gbSearchIn, "cbOtherNames");
  cbOtherNames->setChecked(true);
  cbCaseSensitive = new QCheckBox(tr("Gro-/Kleinschreibung beachten"), gbSearch, "cbCaseSensitive");
  cbCaseSensitive->setChecked(false);
  pbSearch = new QPushButton(tr("Suche starten"), gbSearch, "pbSearch");

  QGroupBox *gbOrder = new QGroupBox(3, QGroupBox::Vertical, tr("Bestellung"), vbLSearch, "gbOrder");
  QGroupBox *gbOrderCount = new QGroupBox(1, QGroupBox::Horizontal, tr("Bisher bestellte Artikel"), gbOrder, "gbOrderCount");
  leOrderCount = new QLineEdit(gbOrderCount, "leOrderCount");
  QGroupBox *gbOrderSum = new QGroupBox(2, QGroupBox::Horizontal, tr("Summe (ohne Verpackung/Versand)"), gbOrder, "gbOrderSum");
  QGroupBox *gbOrderSumNetto = new QGroupBox(1, QGroupBox::Horizontal, tr("Netto"), gbOrderSum, "gbOrderSumNetto");
  leOrderSumNetto = new QLineEdit(gbOrderSumNetto, "leOrderSumNetto");
  QGroupBox *gbOrderSumBrutto = new QGroupBox(1, QGroupBox::Horizontal, tr("Brutto"), gbOrderSum, "gbOrderSumBrutto");
  leOrderSumBrutto = new QLineEdit(gbOrderSumBrutto, "leOrderSumBrutto");
  pbOrderList = new QPushButton(tr("Bestellliste"), gbOrder, "pbOrderList");

  // Initialize list view
  lvParts->addColumn(tr("Artikel"));
  lvParts->addColumn(tr("VPE"));
  lvParts->addColumn(tr("Menge"));
  lvParts->addColumn(tr("Preis"));
  lvParts->addColumn(tr("Status"));
  lvParts->addColumn(tr("Beschreibung"));
  lvParts->addColumn(tr("Weitere Namen"));
  lvParts->addColumn("C", 0);
  lvParts->addColumn("SC", 0);
  lvParts->addColumn("SSC", 0);
  lvParts->addColumn("A", 0);
  lvParts->setColumnAlignment(1, Qt::AlignRight);
  lvParts->setColumnAlignment(2, Qt::AlignRight);
  lvParts->setColumnAlignment(3, Qt::AlignRight);
  lvParts->setColumnAlignment(4, Qt::AlignCenter);
  lvParts->setAllColumnsShowFocus(true);

  popMenu = new QPopupMenu( this );
  popMenu->insertItem( tr("Bestellen"), this, SLOT( slotOrderPart() ) );
  popMenu->insertItem( tr("Info"), this, SLOT( slotShowArticleInfo() ) );
  popMenu->insertSeparator();
  popMenu->insertItem( tr("Abbestellen"), this, SLOT( slotDisorderPart() ) );
  popMenu->insertItem( tr("Alle abbestellen"), this, SLOT( slotDisorderAllParts() ) );

}

/** Sets up all necessary ``connects'' between \a signals and \a slots 
* for the widget. 
*/
void skCentralWidget::setupConnects()
{
   connect(cbCategory, SIGNAL( activated(int) ),
           this, SLOT ( slotCategoryChanged(int) ));
   connect(cbSubCategory, SIGNAL( activated(int) ),
           this, SLOT ( slotSubCategoryChanged(int) ));
   connect(cbSubSubCategory, SIGNAL( activated(int) ),
           this, SLOT ( slotSubSubCategoryChanged(int) ));
   connect(pbSearch, SIGNAL( clicked() ),
           this, SLOT ( slotSearchArticles() ));
   connect(pbOrderList, SIGNAL( clicked() ),
           this, SLOT ( slotOrderedArticles() ));
   connect(lvParts, SIGNAL( doubleClicked(QListViewItem *) ),
           this, SLOT ( slotDisplayArticle(QListViewItem *) ));
   connect(lvParts, SIGNAL( rightButtonClicked(QListViewItem *, const QPoint &, int) ),
           this, SLOT ( slotRightButtonClicked(QListViewItem *, const QPoint &, int) ));
}    

/** Deletes all data, the combo boxes and the list view.
*/
void skCentralWidget::clearAllData()
{
  sclData.clear();
  lvParts->clear();
  cbCategory->clear();
  cbSubCategory->clear();
  cbSubSubCategory->clear();
}

/*--------------------------------------------------------------- Slots */

/** Is called when the category changes. Updates the combo
* boxes for the sub- and subsub-categories accordingly
* and redisplays the articles.
*/
void skCentralWidget::slotCategoryChanged(int index)
{
  if (index < 0)
  {
    lvParts->clear();
    return;
  }

  // `Search' selected?
  if (tr("Suche") == cbCategory->currentText())
  {
    slotSearchArticles();

    return;
  }

  // `Order list' selected?
  if (tr("Bestellliste") == cbCategory->currentText())
  {
    slotOrderedArticles();

    return;
  }

  // Enable sub categories
  cbSubCategory->setEnabled(true);
  cbSubSubCategory->setEnabled(true);

  sclData[index].UpdateCategory(lvParts, cbSubCategory, cbSubSubCategory, index);
}

/** Is called when the sub-category changes. Updates the combo
* box for the subsub-categories accordingly
* and redisplays the articles.
*/
void skCentralWidget::slotSubCategoryChanged(int index)
{
  int categoryIndex = cbCategory->currentItem();

  if ((index < 0 ) || (categoryIndex < 0))
  {
    lvParts->clear();
    return;
  }

  sclData[categoryIndex].UpdateSubCategory(lvParts, cbSubSubCategory,
                                           categoryIndex, index);
}

/** Is called when the subsub-category changes and
* redisplays the articles.
*/
void skCentralWidget::slotSubSubCategoryChanged(int index)
{
  int categoryIndex = cbCategory->currentItem();
  int subcategoryIndex = cbSubCategory->currentItem();

  if ((index < 0) || 
      (categoryIndex < 0) || 
      (subcategoryIndex < 0))
  {
    lvParts->clear();
    return;
  }

  sclData[categoryIndex].UpdateSubSubCategory(lvParts, categoryIndex, subcategoryIndex, index);
}

/** Searches for the given keyword(s) in the articles
* and displays the list of found entries.
*/
void skCentralWidget::slotSearchArticles()
{
  // Is the search requested by the combo box?
  if (tr("Suche") == cbCategory->currentText())
  {
    // Set all combo boxes to `search' state
    cbSubCategory->setCurrentText(tr("Suche"));
    cbSubSubCategory->setCurrentText(tr("Suche"));
    // Disable sub categories
    cbSubCategory->setEnabled(false);
    cbSubSubCategory->setEnabled(false);

    // Clear list view
    lvParts->clear();
  }

  // Is the search string empty?
  if (leSearch->text().simplifyWhiteSpace().isEmpty())
  {
    QMessageBox::information(this, 
                             "SegorKat", 
                             tr("Geben Sie bitte einen Suchstring ein!"),
                             QMessageBox::Ok);
    return;         // Do nothing...
  }

  // Set all combo boxes to `search' state
  cbCategory->setCurrentText(tr("Suche"));
  cbSubCategory->setCurrentText(tr("Suche"));
  cbSubSubCategory->setCurrentText(tr("Suche"));
  // Disable sub categories
  cbSubCategory->setEnabled(false);
  cbSubSubCategory->setEnabled(false);

  // Clear list view
  lvParts->clear();

  // Search articles
  unsigned int index;
  QProgressDialog progress( tr("Suche luft..."), tr("Abbruch"), sclData.size()+1,
                            0, "progress", TRUE );
  progress.setMinimumDuration(0);

  for (index = 0; index < sclData.size(); ++index)
  {
    progress.setProgress( index+1 );
    qApp->processEvents();
    if ( progress.wasCancelled() )
      break;
     
    sclData[index].SearchArticles(lvParts, leSearch->text(),
                                  cbName->isChecked(),
                                  cbDescription->isChecked(),
                                  cbOtherNames->isChecked(),
                                  cbCaseSensitive->isChecked(),
                                  index);
  }
  progress.setProgress( sclData.size()+1 );
}

/** Is called while SegorKat is in ``Search'' or ``OrderList''
* mode. This routine switches back to ``normal'' display
* and sets the selection focus on the article that was
* double-clicked.
*/
void skCentralWidget::slotDisplayArticle(QListViewItem *item)
{
  if (NULL == item)
    return;

  // Is this the normal display and no ``search''?
  if ((unsigned int) cbCategory->currentItem() < sclData.size())
    return;     // Yes, so do nothing...

  // Switch to ``normal'' display and select clicked article
  cbSubCategory->setEnabled(true);
  cbSubSubCategory->setEnabled(true);
  unsigned int categoryIndex = item->text(7).toUInt();
  unsigned int subcategoryIndex = item->text(8).toUInt();
  unsigned int subsubcategoryIndex = item->text(9).toUInt();
  unsigned int articleIndex = item->text(10).toUInt();
  cbCategory->setCurrentItem(categoryIndex);
  sclData[categoryIndex].UpdateCategory(lvParts, 
                                        cbSubCategory,
                                        cbSubSubCategory,
                                        categoryIndex);
  cbSubCategory->setCurrentItem(subcategoryIndex);
  sclData[categoryIndex].UpdateSubCategory(lvParts, 
                                           cbSubSubCategory,
                                           categoryIndex, 
                                           subcategoryIndex);
  cbSubSubCategory->setCurrentItem(subsubcategoryIndex);
  sclData[categoryIndex].UpdateSubSubCategory(lvParts,
                                              categoryIndex, 
                                              subcategoryIndex, 
                                              subsubcategoryIndex);
  // Select article
  QListViewItem *currentItem = lvParts->firstChild();

  while (NULL != currentItem)
  {
    if (currentItem->text(10).toUInt() == articleIndex)
      break;
    currentItem = currentItem->nextSibling();
  }

  if (NULL != currentItem)
    lvParts->setSelected(currentItem, true);

}

/** Pops up a small context menu with entries for
* article info, ordering an article a.s.o.
*/
void skCentralWidget::slotRightButtonClicked(QListViewItem *item, 
                                          const QPoint &point,
                                          int column)
{
  if ((item) && (column >= 0))
    popMenu->popup( point );
}

/** Displays a small dialog with some info data
* about the currently selected article.
*/
void skCentralWidget::slotShowArticleInfo()
{
  // Get currently selected item
  QListViewItem *item = lvParts->selectedItem();

  if (NULL == item)
    return;

  // Get indices from clicked item
  unsigned int categoryIndex = item->text(7).toUInt();
  unsigned int subcategoryIndex = item->text(8).toUInt();
  unsigned int subsubcategoryIndex = item->text(9).toUInt();
  unsigned int articleIndex = item->text(10).toUInt();

  // Display article info
  sclData[categoryIndex].ShowArticleInfo(subcategoryIndex,
                                         subsubcategoryIndex,
                                         articleIndex);
}

/** Displays the list of all articles that are currently
* ordered.
*/
void skCentralWidget::slotOrderedArticles()
{
  // Set all combo boxes to `order list' state
  cbCategory->setCurrentText(tr("Bestellliste"));
  cbSubCategory->setCurrentText(tr("Bestellliste"));
  cbSubSubCategory->setCurrentText(tr("Bestellliste"));
  // Disable sub categories
  cbSubCategory->setEnabled(false);
  cbSubSubCategory->setEnabled(false);

  // Clear list view
  lvParts->clear();

  // Search articles
  unsigned int index;

  for (index = 0; index < sclData.size(); ++index)
  {
    sclData[index].OrderedArticles(lvParts, index);
  }
}

/** Display the order dialog for the currently
* selected article.
*/
void skCentralWidget::slotOrderPart()
{
  // Get currently selected item
  QListViewItem *item = lvParts->selectedItem();

  if (NULL == item)
    return;

  // Get indices from clicked item
  unsigned int categoryIndex = item->text(7).toUInt();
  unsigned int subcategoryIndex = item->text(8).toUInt();
  unsigned int subsubcategoryIndex = item->text(9).toUInt();
  unsigned int articleIndex = item->text(10).toUInt();

  // Get new number of ordered units
  sclData[categoryIndex].OrderArticle(subcategoryIndex,
                                      subsubcategoryIndex,
                                      articleIndex);

  if (tr("Bestellliste") != cbCategory->currentText())
  {
    // Update item in list view
    sclData[categoryIndex].UpdateArticleItem(item, 
                                             categoryIndex,
                                             subcategoryIndex,
                                             subsubcategoryIndex,
                                             articleIndex);
  }
  else
  {
    // Rebuild complete list
    slotOrderedArticles();
  }

  // Update price info
  slotUpdateOrderInfo();
}

/** Removes the current article from the order list,
* i.e. sets the ordered units to zero.
*/
void skCentralWidget::slotDisorderPart()
{
  // Get currently selected item
  QListViewItem *item = lvParts->selectedItem();

  if (NULL == item)
    return;

  // Get indices from clicked item
  unsigned int categoryIndex = item->text(7).toUInt();
  unsigned int subcategoryIndex = item->text(8).toUInt();
  unsigned int subsubcategoryIndex = item->text(9).toUInt();
  unsigned int articleIndex = item->text(10).toUInt();

  // Set ordered units to zero
  sclData[categoryIndex].SetOrderArticle(subcategoryIndex,
                                         subsubcategoryIndex,
                                         articleIndex, 0);

  if (tr("Bestellliste") != cbCategory->currentText())
  {
    // Update item in list view
    sclData[categoryIndex].UpdateArticleItem(item, 
                                             categoryIndex,
                                             subcategoryIndex,
                                             subsubcategoryIndex,
                                             articleIndex);
  }
  else
  {
    // Rebuild the order list for deleting the item...
    slotOrderedArticles();
  }

  // Update price info
  slotUpdateOrderInfo();

}

/** Removes all articles from the order list.
*/
void skCentralWidget::slotDisorderAllParts()
{
  unsigned int index;
  for (index = 0; index < sclData.size(); ++index)
  {
    sclData[index].DisOrderAll();
  }

  // Is the order list displayed?
  if (tr("Bestellliste") == cbCategory->currentText())
  {
    // Yes, so delete it completely
    lvParts->clear();
  }
  else
  {
    // No, so update all displayed items
    QListViewItem *l = lvParts->firstChild();

    // Indices of current item
    unsigned int categoryIndex;
    unsigned int subcategoryIndex;
    unsigned int subsubcategoryIndex;
    unsigned int articleIndex;

    while (l != 0)
    {
      // Get indices of current item
      categoryIndex = l->text(7).toUInt();
      subcategoryIndex = l->text(8).toUInt();
      subsubcategoryIndex = l->text(9).toUInt();
      articleIndex = l->text(10).toUInt();

      // Update item in list view
      sclData[categoryIndex].UpdateArticleItem(l, 
                                               categoryIndex,
                                               subcategoryIndex,
                                               subsubcategoryIndex,
                                               articleIndex);

      // Get next item
      l = l->itemBelow();
    }
  }

  slotUpdateOrderInfo();
}

/** Recomputes the number of ordered articles
* and the price sums.
*/
void skCentralWidget::slotUpdateOrderInfo()
{
  unsigned int index;
  int articles = 0;
  double nettoSum = 0.0;
  double bruttoSum = 0.0;

  for (index = 0; index < sclData.size(); ++index)
  {
    sclData[index].UpdateOrderInfo(&articles, &nettoSum, &bruttoSum);
  }

  leOrderCount->setText(QString::number(articles));
  leOrderSumNetto->setText(QString::number(nettoSum, 'f', 2));
  leOrderSumBrutto->setText(QString::number(bruttoSum, 'f', 2));

}

/** Opens an already uncompressed catalog from a 
* data directory.
*/
void skCentralWidget::slotOpenCatalog()
{
  QString fileName;
  QString dirName = QFileDialog::getExistingDirectory(".");

  if ( !dirName.isNull() ) 
  {                 
    // We got a directory, so open data files...
    clearAllData();

    // Ensure that directory ends with a "/"
    if (dirName.at(dirName.length()-1) != '/')
    {
      dirName += "/";
    }	


    QProgressDialog progress( tr("ffne Katalog..."), tr("Abbruch"), CATALOGFILES + 1,
                              0, "progress", TRUE );
    progress.setMinimumDuration(0);
    progress.setProgress( 1 );
    qApp->processEvents();     

    progress.setLabelText(tr("ffne WGHAUPT"));
    qApp->processEvents();     

    // Read categories
    fileName = dirName + "WGHAUPT";
    ifstream hFile(fileName.latin1());
    skSegorCategory categoryNew;

    if (hFile)
    {
      while (!hFile.eof())
      {
        categoryNew.read(hFile);
        if (!hFile.eof())
        {
          cbCategory->insertItem(QString(categoryNew.Description.c_str()));
          sclData.push_back(categoryNew);
        }
      }
      hFile.close();
    }
    else
    {
      progress.setProgress( CATALOGFILES + 1 );
      QMessageBox::warning(this,
                           "SegorKat", 
                           tr("Die Datei WGHAUPT konnte nicht geffnet werden!"));
      clearAllData();
      return;
    }
    cbCategory->insertItem(tr("Suche"));
    cbCategory->insertItem(tr("Bestellliste"));

    progress.setProgress( 2 );
    qApp->processEvents();     
    if ( progress.wasCancelled() )
    {
      clearAllData();
      return;
    }
    progress.setLabelText(tr("ffne WGOBER"));
    qApp->processEvents();     

    // Read subcategories
    fileName = dirName + "WGOBER";
    ifstream oFile(fileName.latin1());
    skSegorSubCategory subcategoryNew;
    vector<skSegorCategory>::iterator c_it;
    string idPart;

    if (oFile)
    {
      while (!oFile.eof())
      {
        subcategoryNew.read(oFile);
        if (!oFile.eof())
        {
          idPart = subcategoryNew.ID.substr(0, 1);
          for (c_it = sclData.begin(); c_it != sclData.end(); ++c_it)
          {
            if (c_it->ID == idPart)
            {
              c_it->InsertSubCategory(subcategoryNew);
              break;
            }
          }
        }
      }
      oFile.close();
    }
    else
    {
      progress.setProgress( CATALOGFILES + 1 );
      QMessageBox::warning(this,
                           "SegorKat", 
                           tr("Die Datei WGOBER konnte nicht geffnet werden!"));
      clearAllData();
      return;
    }

    progress.setProgress( 3 );
    qApp->processEvents();     
    if ( progress.wasCancelled() )
    {
      clearAllData();
      return;
    }
    progress.setLabelText(tr("ffne WGGRUPPE"));
    qApp->processEvents();     

    // Read subsubcategories
    fileName = dirName + "WGGRUPPE";
    ifstream gFile(fileName.latin1());
    skSegorSubSubCategory subsubcategoryNew;

    if (gFile)
    {
      while (!gFile.eof())
      {
        subsubcategoryNew.read(gFile);
        if (!gFile.eof())
        {
          idPart = subsubcategoryNew.ID.substr(0, 1);
          for (c_it = sclData.begin(); c_it != sclData.end(); ++c_it)
          {
            if (c_it->ID == idPart)
            {
              c_it->InsertSubSubCategory(subsubcategoryNew);
              break;
            }
          }
        }
      }
      gFile.close();
    }
    else
    {
      progress.setProgress( CATALOGFILES + 1 );
      QMessageBox::warning(this,
                           "SegorKat", 
                           tr("Die Datei WGGRUPPE konnte nicht geffnet werden!"));
      clearAllData();
      return;
    }

    progress.setProgress( 4 );
    qApp->processEvents();     
    if ( progress.wasCancelled() )
    {
      clearAllData();
      return;
    }
    progress.setLabelText(tr("ffne ART"));
    qApp->processEvents();     

    // Read articles
    fileName = dirName + "ART";
    ifstream aFile(fileName.latin1());
    skSegorArticle articleNew;

    if (aFile)
    {
      while (!aFile.eof())
      {
        articleNew.read(aFile);
        if (!aFile.eof())
        {
          idPart = articleNew.parentID.substr(0, 1);
          for (c_it = sclData.begin(); c_it != sclData.end(); ++c_it)
          {
            if (c_it->ID == idPart)
            {
              c_it->InsertArticle(articleNew);
              break;
            }
          }
        }
      }
      aFile.close();
    }
    else
    {
      progress.setProgress( CATALOGFILES + 1 );
      QMessageBox::warning(this,
                           "SegorKat", 
                           tr("Die Datei ART konnte nicht geffnet werden!"));
      clearAllData();
      return;
    }

    progress.setProgress( CATALOGFILES + 1 );
    qApp->processEvents();     

    if (sclData.size() > 0)
      sclData[0].UpdateCategory(lvParts,
                                cbSubCategory,
                                cbSubSubCategory,
                                0);
  }

  slotUpdateOrderInfo();
}

/** Uncompresses a catalog to a 
* data directory.
*/
void skCentralWidget::slotUncompressCatalog()
{
  QString fileName = QFileDialog::getOpenFileName(".", 
                                                  tr("Alle Dateien") + " (*.*)");

  if ( !fileName.isNull() ) 
  {                 
    // We got a file name, so get a directory...
    QMessageBox::information(this, tr("SegorKat"),
                             tr("Bitte whlen Sie als nchstes das") + "\n" +
  			     tr("Verzeichnis, in das alle Dateien entpackt") + "\n" +
                             tr("werden sollen."));

    QString dirName = QFileDialog::getExistingDirectory(".");
    if ( !dirName.isNull() ) 
    {                 
      // We got a directory, so uncompress the binary file...

      // Ensure that directory ends with a "/"
      if (dirName.at(dirName.length()-1) != '/')
      {
        dirName += "/";
      }	
      if (ERROR == uncompressSegorCatalog(fileName, dirName))
      {
        QMessageBox::warning(this,
                             "SegorKat", 
                             tr("Die Datei konnte nicht vollstndig entpackt werden!"));
      }
    }
  }
}


/** Constructs a HTML table string for the order list.
*/
void skCentralWidget::getHtmlOrderTable(QString &sTable)
{
  sTable = "";
  sTable += "<CENTER><H2>" + tr("Bestellliste") + "</H2><P>\n";
  sTable += "<TABLE BORDER=1 CELLSPACING=0>\n";
  sTable += "<TR><TH>Menge (VPE)</TH><TH>Artikel</TH><TH>Netto</TH>";
  sTable += "<TH>Brutto</TH></TR>\n";

  // Scan all subcategories
  unsigned int index;

  for (index = 0; index < sclData.size(); ++index)
  {
    sclData[index].getHtmlOrderTable(sTable);
  }

  sTable += "<TR><TD>&nbsp;</TD><TD ALIGN=LEFT><B>Summe</B></TD>";
  sTable += "<TD ALIGN=RIGHT><B>" + leOrderSumNetto->text();
  sTable += "</B></TD><TD ALIGN=RIGHT><B>" + leOrderSumBrutto->text();
  sTable += "</B></TD></TR>\n";

  sTable += "</TABLE></CENTER>\n";
}

/** Constructs a Text table string for the order list.
*/
void skCentralWidget::getTextOrderTable(QString &sTable)
{
  sTable = "";
  // Scan all subcategories
  unsigned int index;
  QString sepString = " | ";
  QChar lineChar = '=';
  int ColCountWidth = 12;
  int ColArticleWidth = 34;
  int ColNettoWidth = 10;
  int ColBruttoWidth = 10;

  addTextCell(sTable, tr("Menge (VPE)"), ColCountWidth, ALIGN_LEFT);
  sTable += sepString;
  addTextCell(sTable, tr("Artikel"), ColArticleWidth, ALIGN_LEFT);
  sTable += sepString;
  addTextCell(sTable, tr("Netto"), ColNettoWidth, ALIGN_LEFT);
  sTable += sepString;
  addTextCell(sTable, tr("Brutto"), ColBruttoWidth, ALIGN_LEFT);
  sTable += "\n";

  int i = 0;
  int lineWidth = ColCountWidth + ColArticleWidth + 
                  ColNettoWidth + ColBruttoWidth + 3*sepString.length();

  for (; i < lineWidth; ++i)
    sTable += lineChar;
  sTable += "\n";

  for (index = 0; index < sclData.size(); ++index)
  {
    sclData[index].getTextOrderTable(sTable, sepString, ColCountWidth, 
                                     ColArticleWidth, ColNettoWidth,
                                     ColBruttoWidth);
  }

  for (i = 0; i < lineWidth; ++i)
    sTable += lineChar;
  sTable += "\n";

  addTextCell(sTable, "", ColCountWidth);
  sTable += sepString;
  addTextCell(sTable, tr("Summe"), ColArticleWidth, ALIGN_LEFT);
  sTable += sepString;
  addTextCell(sTable, leOrderSumNetto->text(), ColNettoWidth);
  sTable += sepString;
  addTextCell(sTable, leOrderSumBrutto->text(), ColBruttoWidth);
  sTable += "\n\n";

}

/** Prints the current list of orders.
*/
void skCentralWidget::slotPrintOrderList()
{
  QPrinter printer;
  printer.setFullPage(TRUE);
  if ( printer.setup( this ) ) 
  {
    QPainter p( &printer );
    QPaintDeviceMetrics metrics(p.device());
    int dpix = metrics.logicalDpiX();
    int dpiy = metrics.logicalDpiY();
    const int margin = 72; // pt
    QRect body(margin*dpix/72, margin*dpiy/72,
               metrics.width()-margin*dpix/72*2,
               metrics.height()-margin*dpiy/72*2 );

    QString rText;
    QString htmlText;
    // Create rich text
    rText = "<QT>\n";
    getHtmlOrderTable(htmlText);
    rText.append(htmlText);
    rText += "</QT>\n";

    QSimpleRichText richText( rText, QFont());
    richText.setWidth( &p, body.width() );
    QRect view( body );
    int page = 1;
    do 
    {
      richText.draw( &p, body.left(), body.top(), view, colorGroup() );
      view.moveBy( 0, body.height() );
      p.translate( 0 , -body.height() );
      p.drawText( view.right() - p.fontMetrics().width( QString::number(page) ),
                  view.bottom() + p.fontMetrics().ascent() + 5, QString::number(page) );
      if ( view.top()  >= richText.height() )
          break;
      printer.newPage();
      page++;
    } while (TRUE);
  }
}

/** Exports the current list of orders into
* an ASCII text file.
*/
void skCentralWidget::slotExportOrderListText()
{
  QString fileName = QFileDialog::getSaveFileName(".",
                                                  tr("Text-Dateien") + " (*.txt)");
  if ( !fileName.isNull() ) 
  {                 
    // We got a file name, so write the file...
    QFile f(fileName);
    if (f.open(IO_WriteOnly))
    {
      QTextStream t( &f );
      QString textData;
 
      // Store data
      getTextOrderTable(textData);
      t << tr("Bestellliste") << endl;
      t <<    "============" << endl << endl;
      t << textData;
      
      f.close();
    }
  }
}

/** Exports the current list of orders into
* a HTML file.
*/
void skCentralWidget::slotExportOrderListHTML()
{
  QString fileName = QFileDialog::getSaveFileName(".",
                                                  tr("HTML-Dateien") + " (*.html)");
  if ( !fileName.isNull() ) 
  {                 
    // We got a file name, so write the file...
    QFile f(fileName);
    if (f.open(IO_WriteOnly))
    {
      QTextStream t( &f );
      QString htmlData;
 
      // Store data
      t << "<HTML>" << endl;
      t << "<HEAD>" << endl;
      t << "<TITLE>" << tr("Bestelliste") << "</TITLE>" << endl;
      t << "</HEAD"  << endl;
      t << "<BODY>" << endl;
      getHtmlOrderTable(htmlData);
      t << htmlData;
      t << "</BODY>" << endl;
      t << "</HTML>" << endl;

      f.close();
    }
  }
}

