/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programs/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skSegorFile.cpp 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

/*------------------------------------------------------------- Includes */

#include "skGlobal.h"
#include "skSegorFile.h"
#include "skFileIO.h"

/*------------------------------------------------------ Class functions */

/** Default constructor for the \a skSegorFile class.
*/
skSegorFile::skSegorFile() : 
	Name(),
        Extension(),
        Compression(0),
        Size(0)
{
  ;
}

/** Copy constructor for the \a skSegorFile class.
* @param elem File info that is copied
*/
skSegorFile::skSegorFile(const skSegorFile &elem) 
{
  Name = elem.Name;
  Extension = elem.Extension;
  Compression = elem.Compression;
  Size = elem.Size;
}

/** Overloaded copy operator ``=''.
* @param elem File info that is copied
*/
skSegorFile &skSegorFile::operator= (const skSegorFile &elem)
{
  if (this == &elem)
    return *this;

  Name = elem.Name;
  Extension = elem.Extension;
  Compression = elem.Compression;
  Size = elem.Size;

  return *this;
}

/** Reads the single entries from the given file stream \a f.
* @param f File stream, the entries are read from
* @return OK if entry was read successfully, ERROR else
*/
int skSegorFile::read(ifstream &f)
{
  char c;

  Name = "";
  Extension = "";

  // Read filename
  int i = 0;
  bool finished = false;
  for (; i < 9; ++i)
  {
    f.get(c);
    if ((0 != c) && (false == finished))
      Name += c;
    else
      finished = true;
  }
  // Read extension
  finished = false;
  for (i = 0; i < 4; ++i)
  {
    f.get(c);
    if ((0 != c) && (false == finished))
      Extension += c;
    else
      finished = true;
  }
  // Read compression type
  f.get(c);
  Compression = (int) c;
  // Read filesize
  Size = readULongInt(f);

  return OK;
}

