/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programs/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skSegorSubSubCategory.h 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

#ifndef SK_SEGORSUBSUBCATEGORY_H
#define SK_SEGORSUBSUBCATEGORY_H

/*------------------------------------------------------------- Includes */

#include "skSegorArticle.h"

#include <iostream>
#include <string>
#include <fstream>
#include <vector>

using namespace std;

/*------------------------------------------------------ Forward class declarations */

#ifdef SEGORKAT_QT
class QListView;
class QListViewItem;
class QComboBox;
class QPixmap;
#endif

/*-------------------------------------------------------------- Classes */

/** Parameters for a subsub-category (=Warengruppe) in the Segor archive. 
*/
class skSegorSubSubCategory
{
public:
    skSegorSubSubCategory();
    skSegorSubSubCategory(const skSegorSubSubCategory &elem);
    skSegorSubSubCategory& operator= (const skSegorSubSubCategory &elem);
    
    int read(ifstream &f);
    void InsertArticle(const skSegorArticle &article);

#ifdef SEGORKAT_QT
    void UpdateArticles(QListView *lvParts,
                        int categoryIndex,
                        int subcategoryIndex,
                        int subsubCategoryIndex);
    void UpdateArticleItem(QListViewItem *lviItem,
                           int categoryIndex,
                           int subcategoryIndex,
                           int subsubCategoryIndex,
                           int articleIndex);

    void SearchArticles(QListView *lvParts, 
                        const QString &search,
                        bool inName,
                        bool inDescription,
                        bool inOtherNames,
                        bool caseSensitive,
                        int categoryIndex,
                        int subcategoryIndex,
                        int subsubcategoryIndex);
    void ShowArticleInfo(int articleIndex);

    void OrderArticle(int articleIndex);
    void OrderedArticles(QListView *lvParts, 
                         int categoryIndex,
                         int subcategoryIndex,
                         int subsubcategoryIndex);

    void getHtmlOrderTable(QString &sTable); 
    void getTextOrderTable(QString &sTable, 
                           const QString &sepString,
                           int ColCountWidth, 
                           int ColArticleWidth, 
                           int ColNettoWidth,
                           int ColBruttoWidth);

#endif
    void SetOrderArticle(int articleIndex, 
                         int orderedUnits);
    void DisOrderAll();

    void UpdateOrderInfo(int *articleCount, 
                         double *nettoSum,
                         double *bruttoSum);

    /** ID of the category */
    string ID;
    /** Description */
    string Description;

protected:

private:
#ifdef SEGORKAT_QT
    void AddArticleToListView(QListView *lvParts,
                              skSegorArticle &article,
                              int categoryIndex,
                              int subcategoryIndex,
                              int subsubcategoryIndex,
                              int articleIndex);
    void AddOrderToListView(QListView *lvParts,
                            skSegorArticle &article,
                            int categoryIndex,
                            int subcategoryIndex,
                            int subsubcategoryIndex,
                            int articleIndex);
    void UpdateArticleData(QListViewItem *lviItem,
                           skSegorArticle &article,
                           int categoryIndex,
                           int subcategoryIndex,
                           int subsubcategoryIndex,
                           int articleIndex);
    void UpdateOrderData(QListViewItem *lviItem,
                         skSegorArticle &article,
                         int categoryIndex,
                         int subcategoryIndex,
                         int subsubcategoryIndex,
                         int articleIndex);
    const QPixmap &GetOKPixmap();
    const QPixmap &GetNilPixmap();

    static QPixmap *okPixmap;
    static QPixmap *nilPixmap;
#endif

    /** List of articles */
    vector<skSegorArticle> alArticles;

};

#endif

